window.Gleam.initSearch([{"doc":"tote","title":"tote","content":"# tote\n\n[![Package Version](https://img.shields.io/hexpm/v/tote)](https://hex.pm/packages/tote)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/tote/)\n\n👜 Bags (or multisets) in Gleam\n\n&gt; ⚙️ This package works for both the Erlang and JavaScript target\n\n## What&#39;s a bag?\n\nA `Bag` is a data structure similar to a set with the difference that it\ncan store multiple copies of the same item.\n\nThis means that you can efficiently check if an element is inside a bag with\nthe [contains](https://hexdocs.pm/tote/tote/bag.html#contains) function,\njust like a set.\nAt the same time, you can ask how many copies of an item are contained inside a\nbag using the [copies](https://hexdocs.pm/tote/tote/bag.html#copies) function.\n\n&gt; If you are curious about implementation details, a `Bag(a)` is nothing\n&gt; more than a handy wrapper around a `Map(a, Int)` that is used to keep\n&gt; track of the number of occurrences of each item.\n\n## Installation\n\nTo add this package to your Gleam project:\n\n```sh\ngleam add tote\n```\n\n## Getting started\n\nAll the bag-related functions are in the `tote/bag` module, so once you&#39;ve\nimported that you&#39;re good to go:\n\n```gleam\nimport gleam/int\nimport gleam/io\nimport tote/bag\n\npub fn main() {\n  let valentino_bag =\n    bag.from_list([\n      &quot;lipstick&quot;, &quot;in&quot;, &quot;my&quot;, &quot;wh-&quot;, &quot;wh-&quot;, &quot;lipstick&quot;,\n      &quot;in&quot;, &quot;my&quot;, &quot;Valentino&quot;, &quot;white&quot;, &quot;bag&quot;, &quot;?!&quot;\n    ])\n\n  bag.copies(of: &quot;lipstick&quot;, in: valentino_bag)\n  |&gt; int.to_string\n  |&gt; io.println\n  // -&gt; 2\n}\n```\n\nIf you don&#39;t get the Valentino reference,\n[you&#39;re welcome!](https://www.youtube.com/watch?v=IzTlPjgJfTk)\n\n## Contributing\n\nIf you think there&#39;s any way to improve this package, or if you spot a bug don&#39;t\nbe afraid to open PRs, issues or requests of any kind! Any contribution is\nwelcome 💜\n","url":"index.html"},{"doc":"tote/bag","title":"Bag","content":"pub opaque type Bag(a)\n A `Bag` is a data structure similar to a set with the difference that it\n can store multiple copies of the same item.\n\n This means that you can efficiently check if an element is inside a bag with\n the [contains](#contains) function, just like a set. At the same time, you\n can ask how many copies of an item are contained inside a bag using the\n [copies](#copies) function.\n\n &gt; If you are curious about implementation details, a `Bag(a)` is nothing\n &gt; more than a handy wrapper around a `Map(a, Int)` that is used to keep\n &gt; track of the number of occurrences of each item.\n\n\nSynonyms:\ntote/bag.Bag\ntote/bag Bag","url":"tote/bag.html#Bag"},{"doc":"tote/bag","title":"contains","content":"pub fn contains(bag: Bag(a), item: a) -&gt; Bool\n Returns `True` if the bag contains at least a copy of the given item.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;])\n |&gt; bag.contains(&quot;a&quot;)\n // True\n \n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;])\n |&gt; bag.contains(&quot;c&quot;)\n // False\n \n\nSynonyms:\ntote/bag.contains\ntote/bag contains","url":"tote/bag.html#contains"},{"doc":"tote/bag","title":"copies","content":"pub fn copies(in bag: Bag(a), of item: a) -&gt; Int\n Counts the number of copies of an item inside a bag.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;, &quot;c&quot;])\n |&gt; bag.copies(of: &quot;a&quot;)\n // 2\n \n\nSynonyms:\ntote/bag.copies\ntote/bag copies","url":"tote/bag.html#copies"},{"doc":"tote/bag","title":"filter","content":"pub fn filter(\n  bag: Bag(a),\n  keeping predicate: fn(a, Int) -&gt; Bool,\n) -&gt; Bag(a)\n Only keeps the items of a bag the respect a given predicate that takes as\n input an item and the number of its copies.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;, &quot;b&quot;, &quot;c&quot;, &quot;d&quot;])\n |&gt; bag.filter(keeping: fn(_, copies) { copies &lt;= 1 })\n |&gt; bag.to_list\n // [#(&quot;c&quot;, 1), #(&quot;d&quot;, 1)]\n \n\nSynonyms:\ntote/bag.filter\ntote/bag filter","url":"tote/bag.html#filter"},{"doc":"tote/bag","title":"fold","content":"pub fn fold(\n  over bag: Bag(a),\n  from initial: b,\n  with fun: fn(b, a, Int) -&gt; b,\n) -&gt; b\n Combines all items of a bag into a single value by calling a given function\n on each one.\n\n The function will receive as input the accumulator, the item and\n its number of copies.\n\n ## Examples\n\n \n let bag = bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;b&quot;])\n bag.fold(over: bag, from: 0, with: fn(count, _, copies) {\n   count + copies\n })\n // 3\n \nSynonyms:\ntote/bag.fold\ntote/bag fold","url":"tote/bag.html#fold"},{"doc":"tote/bag","title":"from_list","content":"pub fn from_list(list: List(a)) -&gt; Bag(a)\n Creates a new bag from the given list by counting its items.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;, &quot;c&quot;])\n |&gt; bag.to_list\n // [#(&quot;a&quot;, 2), #(&quot;b&quot;, 1), #(&quot;c&quot;, 1)]\n \n\nSynonyms:\ntote/bag.from_list\ntote/bag from_list","url":"tote/bag.html#from_list"},{"doc":"tote/bag","title":"from_map","content":"pub fn from_map(map: Dict(a, Int)) -&gt; Bag(a)\n Creates a new bag from the map where each key/value pair is turned into\n an item with those many copies inside the bag.\n\n ## Examples\n\n \n map.from_list([#(&quot;a&quot;, 1), #(&quot;b&quot;, 2)])\n |&gt; bag.from_map\n |&gt; bag.to_list\n // [#(&quot;a&quot;, 1), #(&quot;b&quot;, 2)]\n \n\nSynonyms:\ntote/bag.from_map\ntote/bag from_map","url":"tote/bag.html#from_map"},{"doc":"tote/bag","title":"insert","content":"pub fn insert(\n  into bag: Bag(a),\n  copies to_add: Int,\n  of item: a,\n) -&gt; Bag(a)\n Adds `n` copies of the given item into a bag.\n\n If the number of copies to add is negative, then this is the same as calling\n [`remove`](#remove) and will remove that many copies from the bag.\n\n ## Examples\n\n \n bag.new()\n |&gt; bag.insert(2, &quot;a&quot;)\n |&gt; bag.copies(of: &quot;a&quot;)\n // 2\n \n\n \n bag.from_list([&quot;a&quot;])\n |&gt; bag.insert(-1, &quot;a&quot;)\n |&gt; bag.copies(of: &quot;a&quot;)\n // 0\n \n\nSynonyms:\ntote/bag.insert\ntote/bag insert","url":"tote/bag.html#insert"},{"doc":"tote/bag","title":"intersect","content":"pub fn intersect(one: Bag(a), with other: Bag(a)) -&gt; Bag(a)\n Intersects two bags keeping the minimum number of copies of each item\n that appear in both bags.\n\n ## Examples\n\n \n let bag1 = bag.from_list([&quot;a&quot;, &quot;a&quot;, &quot;b&quot;, &quot;c&quot;])\n let bag2 = bag.from_list([&quot;a&quot;, &quot;c&quot;, &quot;c&quot;])\n\n bag.intersect(bag1, bag2)\n |&gt; bag.to_list\n // [#(&quot;a&quot;, 1), #(&quot;c&quot;, 1)]\n \n\nSynonyms:\ntote/bag.intersect\ntote/bag intersect","url":"tote/bag.html#intersect"},{"doc":"tote/bag","title":"is_empty","content":"pub fn is_empty(bag: Bag(a)) -&gt; Bool\n Returns `True` if the bag is empty.\n\n &gt; ⚠️ This is more efficient than checking if the bag&#39;s size is 0.\n &gt; You should always use this function to check that a bag is empty!\n\n ## Examples\n\n \n bag.new()\n |&gt; bag.is_empty()\n // True\n \n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;])\n |&gt; bag.is_empty()\n // False\n \n\nSynonyms:\ntote/bag.is_empty\ntote/bag is_empty","url":"tote/bag.html#is_empty"},{"doc":"tote/bag","title":"map","content":"pub fn map(bag: Bag(a), with fun: fn(a, Int) -&gt; b) -&gt; Bag(b)\n Updates all values of a bag calling on each a function that takes as\n argument the item and its number of copies.\n\n If one or more items are mapped to the same item, their occurrences are\n summed up.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;b&quot;])\n |&gt; bag.map(fn(item, _) { &quot;c&quot; })\n |&gt; bag.to_list\n // [#(&quot;c&quot;, 3)]\n \n\nSynonyms:\ntote/bag.map\ntote/bag map","url":"tote/bag.html#map"},{"doc":"tote/bag","title":"merge","content":"pub fn merge(one: Bag(a), with other: Bag(a)) -&gt; Bag(a)\n Adds all the items of two bags together.\n\n ## Examples\n\n \n let bag1 = bag.from_list([&quot;a&quot;, &quot;b&quot;])\n let bag2 = bag.from_list([&quot;b&quot;, &quot;c&quot;])\n\n bag.merge(bag1, bag2)\n |&gt; bag.to_list\n // [#(&quot;a&quot;, 1), #(&quot;b&quot;, 2), #(&quot;c&quot;, 1)]\n \n\nSynonyms:\ntote/bag.merge\ntote/bag merge","url":"tote/bag.html#merge"},{"doc":"tote/bag","title":"new","content":"pub fn new() -&gt; Bag(a)\n Creates a new empty bag.\n\nSynonyms:\ntote/bag.new\ntote/bag new","url":"tote/bag.html#new"},{"doc":"tote/bag","title":"remove","content":"pub fn remove(\n  from bag: Bag(a),\n  copies to_remove: Int,\n  of item: a,\n) -&gt; Bag(a)\n Removes `n` copies of the given item from a bag.\n\n If the quantity to remove is greater than the number of copies in the bag,\n all copies of that item are removed.\n\n &gt; ⚠️ Giving a negative quantity to remove doesn&#39;t really make sense, so the\n &gt; sign of the `copies` argument is always ignored.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;a&quot;])\n |&gt; bag.remove(1, &quot;a&quot;)\n |&gt; bag.copies(of: &quot;a&quot;)\n // 1\n \n\n \n bag.from_list([&quot;a&quot;, &quot;a&quot;])\n |&gt; bag.remove(-1, &quot;a&quot;)\n |&gt; bag.copies(of: &quot;a&quot;)\n // 1\n \n\n \n bag.from_list([&quot;a&quot;, &quot;a&quot;])\n |&gt; bag.remove(10, &quot;a&quot;)\n |&gt; bag.copies(of: &quot;a&quot;)\n // 0\n \n\nSynonyms:\ntote/bag.remove\ntote/bag remove","url":"tote/bag.html#remove"},{"doc":"tote/bag","title":"remove_all","content":"pub fn remove_all(from bag: Bag(a), copies_of item: a) -&gt; Bag(a)\n Removes all the copies of a given item from a bag.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;])\n |&gt; bag.remove_all(&quot;a&quot;)\n |&gt; bag.to_list\n // [#(b, 1)]\n \nSynonyms:\ntote/bag.remove_all\ntote/bag remove_all","url":"tote/bag.html#remove_all"},{"doc":"tote/bag","title":"size","content":"pub fn size(bag: Bag(a)) -&gt; Int\n Returns the total number of items inside a bag.\n\n &gt; ⚠️ This function takes linear time in the number of distinct items in the\n &gt; bag.\n &gt;\n &gt; If you need to check that a bag is empty, you should always use the\n &gt; [`is_empty`](#is_empty) function instead of checking if the size is 0.\n &gt; It&#39;s going to be way more efficient!\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;, &quot;c&quot;])\n |&gt; bag.size\n // 4\n \n\nSynonyms:\ntote/bag.size\ntote/bag size","url":"tote/bag.html#size"},{"doc":"tote/bag","title":"subtract","content":"pub fn subtract(\n  from one: Bag(a),\n  items_of other: Bag(a),\n) -&gt; Bag(a)\n Removes all items of the second bag from the first one.\n\n ## Examples\n\n \n let bag1 = bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;b&quot;])\n let bag2 = bag.from_list([&quot;b&quot;, &quot;c&quot;])\n\n bag.subtract(from: one, items_of: other)\n |&gt; bag.to_list\n // [#(&quot;a&quot;, 1), #(&quot;b&quot;, 1)]\n \n\nSynonyms:\ntote/bag.subtract\ntote/bag subtract","url":"tote/bag.html#subtract"},{"doc":"tote/bag","title":"to_list","content":"pub fn to_list(bag: Bag(a)) -&gt; List(#(a, Int))\n Turns a `Bag` into a list of items and their respective number of copies in\n the bag.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;, &quot;c&quot;])\n |&gt; bag.to_list\n // [#(&quot;a&quot;, 2), #(&quot;b&quot;, 1), #(&quot;c&quot;, 1)]\n \n\nSynonyms:\ntote/bag.to_list\ntote/bag to_list","url":"tote/bag.html#to_list"},{"doc":"tote/bag","title":"to_map","content":"pub fn to_map(bag: Bag(a)) -&gt; Dict(a, Int)\n Turns a `Bag` into a map. Each item in the bag becomes a key and the\n associated value is the number of its copies in the bag.\n\nSynonyms:\ntote/bag.to_map\ntote/bag to_map","url":"tote/bag.html#to_map"},{"doc":"tote/bag","title":"to_set","content":"pub fn to_set(bag: Bag(a)) -&gt; Set(a)\n Turns a `Bag` into a set of its items, losing all information on their\n number of copies.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;, &quot;b&quot;, &quot;a&quot;, &quot;c&quot;])\n |&gt; bag.to_set\n // set.from_list([&quot;a&quot;, &quot;b&quot;, &quot;c&quot;])\n \n\nSynonyms:\ntote/bag.to_set\ntote/bag to_set","url":"tote/bag.html#to_set"},{"doc":"tote/bag","title":"update","content":"pub fn update(\n  in bag: Bag(a),\n  item item: a,\n  with fun: fn(Int) -&gt; Int,\n) -&gt; Bag(a)\n Updates the number of copies of an item in the bag.\n\n If the function returns 0 or a negative number, the item is removed from\n the bag.\n\n ## Examples\n\n \n bag.from_list([&quot;a&quot;])\n |&gt; bag.update(&quot;a&quot;, fn(n) { n + 1 })\n |&gt; bag.copies(of: &quot;a&quot;)\n // 2\n \n\n \n bag.new()\n |&gt; bag.update(&quot;a&quot;, fn(_) { 10 })\n |&gt; bag.copies(of: &quot;a&quot;)\n // 10\n \n\n \n bag.from_list([&quot;a&quot;])\n |&gt; bag.update(&quot;a&quot;, fn(_) { -1 })\n |&gt; bag.copies(of: &quot;a&quot;)\n // 0\n \n\nSynonyms:\ntote/bag.update\ntote/bag update","url":"tote/bag.html#update"},{"doc":"tote/bag","title":"tote/bag","content":"\n This module has all the functions you may need to work with bags (if you\n feel something is missing, please\n [open an issue](https://github.com/giacomocavalieri/tote/issues))!\n\n To quickly browse the documentation you can use this cheatsheet:\n\n &lt;table&gt;\n &lt;tbody&gt;\n   &lt;tr&gt;\n     &lt;td&gt;Creating bags&lt;/td&gt;\n     &lt;td&gt;\n      &lt;a href=&quot;#new&quot;&gt;new&lt;/a&gt;,\n      &lt;a href=&quot;#from_list&quot;&gt;from_list&lt;/a&gt;,\n      &lt;a href=&quot;#from_map&quot;&gt;from_map&lt;/a&gt;\n     &lt;/td&gt;\n   &lt;/tr&gt;\n   &lt;tr&gt;\n     &lt;td&gt;Adding or removing items&lt;/td&gt;\n     &lt;td&gt;\n      &lt;a href=&quot;#insert&quot;&gt;insert&lt;/a&gt;,\n      &lt;a href=&quot;#remove&quot;&gt;remove&lt;/a&gt;,\n      &lt;a href=&quot;#remove_all&quot;&gt;remove_all&lt;/a&gt;,\n      &lt;a href=&quot;#update&quot;&gt;update&lt;/a&gt;\n     &lt;/td&gt;\n   &lt;/tr&gt;\n   &lt;tr&gt;\n     &lt;td&gt;Querying the content of a bag&lt;/td&gt;\n     &lt;td&gt;\n      &lt;a href=&quot;#copies&quot;&gt;copies&lt;/a&gt;,\n      &lt;a href=&quot;#contains&quot;&gt;contains&lt;/a&gt;,\n      &lt;a href=&quot;#is_empty&quot;&gt;is_empty&lt;/a&gt;,\n      &lt;a href=&quot;#size&quot;&gt;size&lt;/a&gt;\n     &lt;/td&gt;\n   &lt;/tr&gt;\n   &lt;tr&gt;\n     &lt;td&gt;Combining bags&lt;/td&gt;\n     &lt;td&gt;\n      &lt;a href=&quot;#intersect&quot;&gt;intersect&lt;/a&gt;,\n      &lt;a href=&quot;#merge&quot;&gt;merge&lt;/a&gt;,\n      &lt;a href=&quot;#subtract&quot;&gt;subtract&lt;/a&gt;\n     &lt;/td&gt;\n   &lt;/tr&gt;\n   &lt;tr&gt;\n     &lt;td&gt;Transforming the content of a bag&lt;/td&gt;\n     &lt;td&gt;\n      &lt;a href=&quot;#fold&quot;&gt;fold&lt;/a&gt;,\n      &lt;a href=&quot;#map&quot;&gt;map&lt;/a&gt;,\n      &lt;a href=&quot;#filter&quot;&gt;filter&lt;/a&gt;\n     &lt;/td&gt;\n   &lt;/tr&gt;\n   &lt;tr&gt;\n     &lt;td&gt;Converting a bag into other data structures&lt;/td&gt;\n     &lt;td&gt;\n      &lt;a href=&quot;#to_list&quot;&gt;to_list&lt;/a&gt;,\n      &lt;a href=&quot;#to_set&quot;&gt;to_set&lt;/a&gt;,\n      &lt;a href=&quot;#to_map&quot;&gt;to_map&lt;/a&gt;\n     &lt;/td&gt;\n   &lt;/tr&gt;\n &lt;/tbody&gt;\n &lt;/table&gt;\n","url":"tote/bag.html"}]);